﻿Option Strict On
Option Explicit On

Imports ESRI.ArcGIS.ArcMapUI
Imports ESRI.ArcGIS.Carto
Imports ESRI.ArcGIS.Geodatabase
Imports ESRI.ArcGIS.esriSystem
Imports ESRI.ArcGIS.Framework
Imports ESRI.ArcGIS.Geometry
Imports ESRI.ArcGIS.Display
Imports System.Math
Public Class ECEditExpressions
    Public Function edit_Generalize() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dTol As Double = CDbl(params.Par1)
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                If pGeometry.GeometryType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline Or _
                   pGeometry.GeometryType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                    Dim pCurve As IPolycurve = CType(pGeometry, IPolycurve)
                    pCurve.Generalize(dTol)
                    Return CType(pCurve, IGeometry)
                Else
                    Return Nothing
                End If
            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_Densify() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dTol As Double = CDbl(params.Par1)
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                If pGeometry.GeometryType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline Or _
                   pGeometry.GeometryType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                    Dim pCurve As IPolycurve = CType(pGeometry, IPolycurve)
                    pCurve.Densify(dTol, 0)
                    Return CType(pCurve, IGeometry)
                Else
                    Return Nothing
                End If
            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_SimplifyGeometry() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dTol As Double = CDbl(params.Par1)
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                Dim pTopOp As ITopologicalOperator2 = CType(pGeometry, ITopologicalOperator2)
                pTopOp.IsKnownSimple_2 = False
                pTopOp.Simplify()
                Return CType(pTopOp, IGeometry)
            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_ClosePolylines() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dTol As Double = CDbl(params.Par1)
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                Dim pPolyline As IPolyline = CType(pGeometry, IPolyline)
                If Not pPolyline.IsClosed Then
                    Dim pLine As ILine = New Line
                    pLine.PutCoords(pPolyline.ToPoint, pPolyline.FromPoint)
                    If pLine.Length < dTol Or pLine.Length > pPolyline.Length Then
                        Dim pSegColl As ISegmentCollection = CType(pPolyline, ISegmentCollection)
                        pSegColl.AddSegment(CType(pLine, ISegment))
                        Return CType(pSegColl, IGeometry)
                    Else
                        Return pGeometry
                    End If
                Else
                    Return pGeometry
                End If
            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_PolygonFillHoles() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim i As Integer
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                Dim pPolygon As IPolygon4 = CType(pGeometry, IPolygon4)
                Dim pGColl As IGeometryCollection = CType(pPolygon, IGeometryCollection)
                Dim iPartsCount As Integer = pGColl.GeometryCount
                Dim iRingsCount = pPolygon.ExteriorRingCount
                If iPartsCount <> iRingsCount Then
                    Dim pRingColl As IGeometryCollection = CType(pPolygon.ExteriorRingBag, IGeometryCollection)
                    Dim pNewPol As IGeometryCollection = New Polygon
                    For i = 0 To pRingColl.GeometryCount - 1
                        pNewPol.AddGeometry(pRingColl.Geometry(i))
                    Next
                    Return CType(pNewPol, IGeometry)
                Else
                    Return pGeometry
                End If
            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_PolygonScale() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dScaleFactorX As Double = CDbl(params.Par1)
            Dim dScaleFactorY As Double = CDbl(params.Par2)
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                Dim pPolygon As IPolygon4 = CType(pGeometry, IPolygon4)
                Dim pArea As IArea = CType(pPolygon, IArea)
                Dim pCenter As IPoint = pArea.Centroid
                Dim pTransform As ITransform2D = CType(pPolygon, ITransform2D)
                With pTransform
                    .Scale(pCenter, dScaleFactorX, dScaleFactorY)
                End With
                Return pPolygon
            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_PolygonRotate() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dAngle As Double = CDbl(params.Par1)
            Dim Pi As Double = 4 * Atan(1)
            dAngle = dAngle * Pi / 180
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                Dim pPolygon As IPolygon4 = CType(pGeometry, IPolygon4)
                Dim pArea As IArea = CType(pPolygon, IArea)
                Dim pCenter As IPoint = pArea.Centroid
                Dim pTransform As ITransform2D = CType(pPolygon, ITransform2D)
                With pTransform
                    .Rotate(pCenter, dAngle)
                End With
                Return pPolygon
            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_PolylineFlip() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                Dim pPolyline As IPolyline = CType(pGeometry, IPolyline)
                pPolyline.ReverseOrientation()
                Return pPolyline
            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_PolylineRotate() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dDistance As Double = CDbl(params.Par1)
            Dim dAngle As Double = CDbl(params.Par2)
            Dim Pi As Double = 4 * Atan(1)
            dAngle = dAngle * Pi / 180
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                Dim pPolyline As IPolyline = CType(pGeometry, IPolyline)
                Dim pPoint As IPoint = New Point
                pPolyline.QueryPoint(0, dDistance, True, pPoint)
                Dim pTransform As ITransform2D = CType(pPolyline, ITransform2D)
                With pTransform
                    .Rotate(pPoint, dAngle)
                End With
                Return pPolyline
            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_PolylineOffset() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dOffsetDistance As Double = CDbl(params.Par1)
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                Dim pPolyline As IPolyline = CType(pGeometry, IPolyline)
                Dim pConstructCurve As IConstructCurve = New Polyline
                pConstructCurve.ConstructOffset(pPolyline, dOffsetDistance)
                Return CType(pConstructCurve, IGeometry)
            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_OffsetZs() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dOffsetDistance As Double = CDbl(params.Par1)
            Dim pZA As IZAware
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                pZA = CType(pGeometry, IZAware)
                If pZA.ZAware Then
                    Dim pZColl As IZCollection = CType(pGeometry, IZCollection)
                    pZColl.OffsetZs(dOffsetDistance)
                    Return CType(pZColl, IGeometry)
                Else
                    Return Nothing
                End If

            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_MultiplyZs() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dMultiplyValue As Double = CDbl(params.Par1)
            Dim pZA As IZAware
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                pZA = CType(pGeometry, IZAware)
                If pZA.ZAware Then
                    Dim pZColl As IZCollection = CType(pGeometry, IZCollection)
                    pZColl.MultiplyZs(dMultiplyValue)
                    Return CType(pZColl, IGeometry)
                Else
                    Return Nothing
                End If

            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_SetConstantZ() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dZValue As Double = CDbl(params.Par1)
            Dim pZA As IZAware
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                pZA = CType(pGeometry, IZAware)
                If pZA.ZAware Then
                    Dim pZ As IZ = CType(pGeometry, IZ)
                    pZ.SetConstantZ(dZValue)
                    Return CType(pZ, IGeometry)
                Else
                    Return Nothing
                End If

            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_OffsetMs() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dOffsetDistance As Double = CDbl(params.Par1)
            Dim pMA As IMAware
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                pMA = CType(pGeometry, IMAware)
                If pMA.MAware Then
                    Dim pMColl As IMCollection = CType(pGeometry, IMCollection)
                    pMColl.OffsetMs(dOffsetDistance)
                    Return CType(pMColl, IGeometry)
                Else
                    Return Nothing
                End If

            End If
        Catch
            Return Nothing
        End Try
    End Function
    Public Function edit_MultiplyMs() As IGeometry
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dMultiplyValue As Double = CDbl(params.Par1)
            Dim pMA As IMAware
            If pGeometry.IsEmpty Then
                Return Nothing
            Else
                pMA = CType(pGeometry, IMAware)
                If pMA.MAware Then
                    Dim pMColl As IMCollection = CType(pGeometry, IMCollection)
                    pMColl.MultiplyMs(dMultiplyValue)
                    Return CType(pMColl, IGeometry)
                Else
                    Return Nothing
                End If
            End If
        Catch
            Return Nothing
        End Try
    End Function
End Class
