﻿Option Strict On
Option Explicit On

Imports ESRI.ArcGIS.ArcMapUI
Imports ESRI.ArcGIS.Carto
Imports ESRI.ArcGIS.Geodatabase
Imports ESRI.ArcGIS.esriSystem
Imports ESRI.ArcGIS.Framework
Imports ESRI.ArcGIS.Geometry
Imports ESRI.ArcGIS.Display
Imports System.Math
Public Class ECGeometryExpressions
    Public Function shape_CountVertices() As Long
        Try
            Dim pGeometry As IGeometry = params.Geometry
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPoint Then
                    Return 1
                End If
                Dim pColl As IPointCollection = CType(pGeometry, IPointCollection)
                Return pColl.PointCount
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function point_Get_X_DMS() As String
        Try
            Dim pPoint As IPoint
            Dim dX As Double

            Dim pGeometry As IGeometry = params.Geometry
            If Not TypeOf pGeometry Is IPoint Then
                Return String.Empty
            End If
            If pGeometry.IsEmpty Then
                Return String.Empty
            End If
            Dim pSR As ISpatialReference = params.DataSref
            If TypeOf pSR Is IProjectedCoordinateSystem Then
                Dim pPCS As IProjectedCoordinateSystem = CType(pSR, IProjectedCoordinateSystem)
                Dim pGCS As ISpatialReference = pPCS.GeographicCoordinateSystem
                pGeometry.SpatialReference = pSR
                pGeometry.Project(pGCS)
            End If
            Dim sType As String = UCase(params.Par1)
            If Not sType = "DM" Then
                sType = "DMS"
            End If
            Dim iNumDec As Short
            If Not IsNumeric(params.Par2) Then
                iNumDec = 2
            Else
                iNumDec = CShort(params.Par2)
            End If
            pPoint = CType(pGeometry, IPoint)
            dX = pPoint.X
            Return EC_DD2DMS(dX, "Lon", sType, iNumDec)
        Catch
            Return String.Empty
        End Try
    End Function
    Public Function point_Get_Y_DMS() As String
        Try
            Dim pPoint As IPoint
            Dim dY As Double
            Dim pGeometry As IGeometry = params.Geometry
            If Not TypeOf pGeometry Is IPoint Then
                Return String.Empty
            End If
            If pGeometry.IsEmpty Then
                Return String.Empty
            End If
            Dim pSR As ISpatialReference = params.DataSref
            If TypeOf pSR Is IProjectedCoordinateSystem Then
                Dim pPCS As IProjectedCoordinateSystem = CType(pSR, IProjectedCoordinateSystem)
                Dim pGCS As ISpatialReference = pPCS.GeographicCoordinateSystem
                pGeometry.SpatialReference = pSR
                pGeometry.Project(pGCS)
            End If
            Dim sType As String = UCase(params.Par1)
            If Not sType = "DM" Then
                sType = "DMS"
            End If
            Dim iNumDec As Short
            If Not IsNumeric(params.Par2) Then
                iNumDec = 2
            Else
                iNumDec = CShort(params.Par2)
            End If
            pPoint = CType(pGeometry, IPoint)
            dY = pPoint.Y
            Return EC_DD2DMS(dY, "Lon", sType, iNumDec)
        Catch
            Return String.Empty
        End Try
    End Function
    Public Function point_Get_M() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPoint Then
                    Return 0
                End If
                Dim pPoint As IPoint = CType(pGeometry, IPoint)
                Dim dM As Double
                Dim pMAware As IMAware = CType(pPoint, IMAware)
                If (pMAware.MAware) Then
                    dM = pPoint.M
                Else
                    dM = 0
                End If
                Return dM
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function point_Get_Z() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPoint Then
                    Return 0
                End If
                Dim pPoint As IPoint = CType(pGeometry, IPoint)
                Dim dZ As Double
                Dim pZAware As IZAware = CType(pPoint, IZAware)
                If (pZAware.ZAware) Then
                    dZ = pPoint.M
                Else
                    dZ = 0
                End If
                Return dZ
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function point_Get_X() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPoint Then
                    Return 0
                End If
                Dim pPoint As IPoint = CType(pGeometry, IPoint)
                Dim dX As Double
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pPoint.SpatialReference = pSrefData
                    pPoint.Project(pSrefMap)
                End If
                dX = pPoint.X
                Return dX
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function point_Get_Y() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPoint Then
                    Return 0
                End If
                Dim pPoint As IPoint = CType(pGeometry, IPoint)
                Dim dY As Double
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pPoint.SpatialReference = pSrefData
                    pPoint.Project(pSrefMap)
                End If
                dY = pPoint.Y
                Return dY
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function polygon_CountTrueParts() As Long
        Try
            Dim pGeometry As IGeometry = params.Geometry
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                    Return 0
                End If
                Dim pPolygon As IPolygon = CType(pGeometry, IPolygon)
                Dim lRingCount As Long = pPolygon.ExteriorRingCount
                Return lRingCount
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function polygon_Get_X_Center() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                    Return 0
                End If
                Dim pPolygon As IPolygon = CType(pGeometry, IPolygon)
                Dim pArea As IArea = CType(pPolygon, IArea)
                Dim pCenter As IPoint = pArea.Centroid
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pCenter.SpatialReference = pSrefData
                    pCenter.Project(pSrefMap)
                End If
                Dim dXCenter As Double = pCenter.X
                Return dXCenter
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function polygon_Get_Y_Center() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                    Return 0
                End If
                Dim pPolygon As IPolygon = CType(pGeometry, IPolygon)
                Dim pArea As IArea = CType(pPolygon, IArea)
                Dim pCenter As IPoint = pArea.Centroid
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pCenter.SpatialReference = pSrefData
                    pCenter.Project(pSrefMap)
                End If
                Dim dYCenter As Double = pCenter.Y
                Return dYCenter
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function polygon_Get_X_Label() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                    Return 0
                End If
                Dim pPolygon As IPolygon = CType(pGeometry, IPolygon)
                Dim pArea As IArea = CType(pPolygon, IArea)
                Dim pCenter As IPoint = pArea.LabelPoint
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pCenter.SpatialReference = pSrefData
                    pCenter.Project(pSrefMap)
                End If
                Dim dXLabel As Double = pCenter.X
                Return dXLabel
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function polygon_Get_Y_Label() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                    Return 0
                End If
                Dim pPolygon As IPolygon = CType(pGeometry, IPolygon)
                Dim pArea As IArea = CType(pPolygon, IArea)
                Dim pCenter As IPoint = pArea.LabelPoint
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pCenter.SpatialReference = pSrefData
                    pCenter.Project(pSrefMap)
                End If
                Dim dYLabel As Double = pCenter.Y
                Return dYLabel
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function polygon_Return_Area() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim sOutUnits As String = params.Par1
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            Dim pCalcSref As ISpatialReference
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                    Return 0
                End If

                Dim pPolygon As IPolygon = CType(pGeometry, IPolygon)
                Dim pArea As IArea = CType(pPolygon, IArea)
                Dim pCenter As IPoint = pArea.LabelPoint
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pCenter.SpatialReference = pSrefData
                    pCenter.Project(pSrefMap)
                    pCalcSref = pSrefMap
                Else
                    pCalcSref = pSrefData
                End If
                Dim dArea As Double = pArea.Area
                Dim dAreaOut As Double = pArea.Area
                If TypeOf pSrefData Is IProjectedCoordinateSystem Then
                    Dim pPCS As IProjectedCoordinateSystem = CType(pCalcSref, IProjectedCoordinateSystem)
                    Dim dMU As Double = pPCS.CoordinateUnit.MetersPerUnit

                    Dim dAreaMeters As Double = dArea * dMU * dMU
                    dAreaOut = EC_ConvertArea(dAreaMeters, sOutUnits)
                Else
                    dAreaOut = dArea
                End If
                Return dAreaOut
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function polygon_Thickness() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            If pGeometry.IsEmpty Then
                Return -1
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                    Return -1
                End If
                Dim pPolygon As IPolygon = CType(pGeometry, IPolygon)
                Dim pArea As IArea = CType(pPolygon, IArea)
                Dim dArea As Double = pArea.Area
                Dim Pi As Double = 4 * Atan(1)
                Dim dRatio As Double
                Dim dLength As Double = pPolygon.Length
                If (dLength > 0) Then
                    dRatio = 4 * Pi * dArea / (dLength * dLength)
                Else
                    dRatio = -1
                End If
                Return dRatio
            End If
        Catch
            Return -1
        End Try
    End Function
    Public Function polyline_Get_Azimuth() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dDistance As Double = CDbl(params.Par1)
            If pGeometry.IsEmpty Then
                Return -1
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline Then
                    Return -1
                End If
                Dim pPolyline As ICurve = CType(pGeometry, ICurve)
                Dim Pi As Double = 4 * Atan(1)
                Dim pLine As ESRI.ArcGIS.Geometry.ILine = New ESRI.ArcGIS.Geometry.Line
                Dim dLength As Double = pPolyline.Length
                pPolyline.QueryTangent(0, dDistance, True, dLength, pLine)
                Dim dAngle As Double = pLine.Angle * 360 / (2 * Pi)
                If (dAngle < 90) Then
                    dAngle = 90 - dAngle
                Else
                    dAngle = 450 - dAngle
                End If
                Return dAngle
            End If
        Catch
            Return -1
        End Try
    End Function
    Public Function polyline_Get_X_AtPoint() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            Dim dDistance As Double = CDbl(params.Par1)
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline Then
                    Return 0
                End If
                Dim pPolyline As ICurve = CType(pGeometry, ICurve)
                Dim pPoint As IPoint = New Point
                pPolyline.QueryPoint(0, dDistance, True, pPoint)
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pPoint.SpatialReference = pSrefData
                    pPoint.Project(pSrefMap)
                End If
                Dim dX As Double = pPoint.X
                Return dX
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function polyline_Get_Y_AtPoint() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            Dim dDistance As Double = CDbl(params.Par1)
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline Then
                    Return 0
                End If
                Dim pPolyline As ICurve = CType(pGeometry, ICurve)
                Dim pPoint As IPoint = New Point
                pPolyline.QueryPoint(0, dDistance, True, pPoint)
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pPoint.SpatialReference = pSrefData
                    pPoint.Project(pSrefMap)
                End If
                Dim dY As Double = pPoint.Y
                Return dY
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function polyline_Get_Z_AtPoint() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dDistance As Double = CDbl(params.Par1)
            Dim pZA As IZAware
            Dim dZ As Double
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline Then
                    Return 0
                End If
                Dim pPolyline As ICurve = CType(pGeometry, ICurve)
                Dim pPoint As IPoint = New Point
                pPolyline.QueryPoint(0, dDistance, True, pPoint)
                pZA = CType(pPolyline, IZAware)
                If pZA.ZAware Then
                    If System.Double.IsNaN(pPoint.Z) Then
                        dZ = 0
                    Else
                        dZ = pPoint.Z
                    End If
                Else
                    dZ = 0
                End If
                Return dZ
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function polyline_Get_M_AtPoint() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim dDistance As Double = CDbl(params.Par1)
            Dim pMA As IMAware
            Dim dM As Double
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline Then
                    Return 0
                End If
                Dim pPolyline As ICurve = CType(pGeometry, ICurve)
                Dim pPoint As IPoint = New Point
                pPolyline.QueryPoint(0, dDistance, True, pPoint)
                pMA = CType(pPolyline, IMAware)
                If pMA.MAware Then
                    If System.Double.IsNaN(pPoint.M) Then
                        dM = 0
                    Else
                        dM = pPoint.M
                    End If
                Else
                    dM = 0
                End If
                Return dM
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function shape_CountAllParts() As Long
        Try
            Dim pGeometry As IGeometry = params.Geometry
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If pGeometry.GeometryType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPoint Then
                    Return 1
                End If
                Dim pGColl As IGeometryCollection = CType(pGeometry, IGeometryCollection)
                Return pGColl.GeometryCount
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function shape_Get_X_Min() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            Dim dDistance As Double = CDbl(params.Par1)
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pGeometry.SpatialReference = pSrefData
                    pGeometry.Project(pSrefMap)
                End If
                Dim pEnv As IEnvelope = pGeometry.Envelope
                If pEnv Is Nothing Then
                    Return 0
                Else
                    Return pEnv.LowerLeft.X
                End If
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function shape_Get_X_Max() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            Dim dDistance As Double = CDbl(params.Par1)
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pGeometry.SpatialReference = pSrefData
                    pGeometry.Project(pSrefMap)
                End If
                Dim pEnv As IEnvelope = pGeometry.Envelope
                If pEnv Is Nothing Then
                    Return 0
                Else
                    Return pEnv.UpperRight.X
                End If
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function shape_Get_Y_Min() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            Dim dDistance As Double = CDbl(params.Par1)
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pGeometry.SpatialReference = pSrefData
                    pGeometry.Project(pSrefMap)
                End If
                Dim pEnv As IEnvelope = pGeometry.Envelope
                If pEnv Is Nothing Then
                    Return 0
                Else
                    Return pEnv.LowerLeft.Y
                End If
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function shape_Get_Y_Max() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            Dim dDistance As Double = CDbl(params.Par1)
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pGeometry.SpatialReference = pSrefData
                    pGeometry.Project(pSrefMap)
                End If
                Dim pEnv As IEnvelope = pGeometry.Envelope
                If pEnv Is Nothing Then
                    Return 0
                Else
                    Return pEnv.UpperRight.Y
                End If
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function shape_Get_M_Min() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pMA As IMAware
            Dim dM As Double
            If pGeometry.IsEmpty Then
                Return 0
            Else
                pMA = CType(pGeometry, IMAware)
                If pMA.MAware Then
                    Dim pMColl As IMCollection = CType(pGeometry, IMCollection)
                    If System.Double.IsNaN(pMColl.MMin) Then
                        dM = 0
                    Else
                        dM = pMColl.MMin
                    End If
                Else
                    dM = 0
                End If
                Return dM
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function shape_Get_M_Max() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pMA As IMAware
            Dim dM As Double
            If pGeometry.IsEmpty Then
                Return 0
            Else
                pMA = CType(pGeometry, IMAware)
                If pMA.MAware Then
                    Dim pMColl As IMCollection = CType(pGeometry, IMCollection)
                    If System.Double.IsNaN(pMColl.MMax) Then
                        dM = 0
                    Else
                        dM = pMColl.MMax
                    End If
                Else
                    dM = 0
                End If
                Return dM
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function shape_Get_Z_Min() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pZA As IZAware
            Dim dZ As Double
            If pGeometry.IsEmpty Then
                Return 0
            Else
                pZA = CType(pGeometry, IZAware)
                If pZA.ZAware Then
                    Dim pZColl As IZCollection = CType(pGeometry, IZCollection)
                    If System.Double.IsNaN(pZColl.ZMin) Then
                        dZ = 0
                    Else
                        dZ = pZColl.ZMin
                    End If
                Else
                    dZ = 0
                End If
                Return dZ
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function shape_Get_Z_Max() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim pZA As IZAware
            Dim dZ As Double
            If pGeometry.IsEmpty Then
                Return 0
            Else
                pZA = CType(pGeometry, IZAware)
                If pZA.ZAware Then
                    Dim pZColl As IZCollection = CType(pGeometry, IZCollection)
                    If System.Double.IsNaN(pZColl.ZMax) Then
                        dZ = 0
                    Else
                        dZ = pZColl.ZMax
                    End If
                Else
                    dZ = 0
                End If
                Return dZ
            End If
        Catch
            Return 0
        End Try
    End Function
    Public Function shape_Get_Length() As Double
        Try
            Dim pGeometry As IGeometry = params.Geometry
            Dim sOutUnits As String = params.Par1
            Dim pSrefMap As ISpatialReference = params.MapSref
            Dim pSrefData As ISpatialReference = params.DataSref
            Dim bDataSref As Boolean = params.SrefFromData
            'if the spatial reference of the map is to be used
            'check whether the data can be projected
            'Only basic check performed below (if one of the spatial references is unknown the data cannot be projected).
            'TO DO - check whether the two spatial references have the same GCS
            If Not bDataSref Then
                If TypeOf pSrefMap Is IUnknownCoordinateSystem Or TypeOf pSrefData Is IUnknownCoordinateSystem Then
                    bDataSref = True
                End If
            End If
            If pGeometry.IsEmpty Then
                Return 0
            Else
                If Not (pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Or _
                        pGeometry.GeometryType <> ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline) Then
                    Return 0
                End If
                Dim pPolycurve As IPolycurve = CType(pGeometry, IPolycurve)
                'If the Map spatial refernce is to be used - project the data
                If Not bDataSref Then
                    pPolycurve.SpatialReference = pSrefData
                    pPolycurve.Project(pSrefMap)
                End If

                Return pPolycurve.Length
            End If
        Catch
            Return 0
        End Try
    End Function
End Class
