﻿Option Strict On
Option Explicit On

Module ExpressionsList
    'this module has 2 dictionaries per category
    'The first dictionary contains the functions together with the parameters needed by each function
    ''''The parameters are represented by a single string and a separated with "|"
    ''''Substring 1 defines the geometry type
    ''''Substring 2 defines the field type
    ''''Substring 3 defines whether the expression requires spatial reference - valid values Y and N
    ''''Substring 4 defines the number of parameters needed. Currently up to 3 parameters can be used
    ''''Each substring 5, 6, 7 defines the names of the parameters to be displayed in the dialog
    'The second dictionary contains the function names together with the description to be displayed to the user.
    'Note that the name of the function in both dictionaries must be exactly the same as the name of the function
    Public Function GetExpressionsCategories() As Dictionary(Of String, String)
        Dim expCat As Dictionary(Of String, String) = New Dictionary(Of String, String)
        Try
            expCat.Add("Geometry", "")
            expCat.Add("Draw", "")
            expCat.Add("Edit", "")
            'to create a new category just add it to the dictionary
            'expCat.Add("My Category", "")
            Return expCat
        Catch
            Return Nothing
        End Try
    End Function
    Public Function GetGeometryExpressions() As Dictionary(Of String, String)
        Dim expDictionary As Dictionary(Of String, String) = New Dictionary(Of String, String)
        Try
            expDictionary.Add("point_Get_X_DMS", "Point|String|N|2|Output Type|Precision")
            expDictionary.Add("point_Get_Y_DMS", "Point|String|N|2|Output Type|Precision")
            expDictionary.Add("point_Get_M", "Point|Number|N|0")
            expDictionary.Add("point_Get_X", "Point|Number|Y|0")
            expDictionary.Add("point_Get_Y", "Point|Number|Y|0")
            expDictionary.Add("point_Get_Z", "Point|Number|N|0")
            expDictionary.Add("polygon_CountTrueParts", "Polygon|Nu  mber|N|0")
            expDictionary.Add("polygon_Get_X_Center", "Polygon|Number|Y|0")
            expDictionary.Add("polygon_Get_Y_Center", "Polygon|Number|Y|0")
            expDictionary.Add("polygon_Get_X_Label", "Polygon|Number|Y|0")
            expDictionary.Add("polygon_Get_Y_Label", "Polygon|Number|Y|0")
            expDictionary.Add("polygon_Return_Area", "Polygon|Number|Y|1|Output Units")
            expDictionary.Add("polygon_Thickness", "Polygon|Number|N|0")
            expDictionary.Add("polyline_Get_Azimuth", "Polyline|Number|N|1|Point Along (0  to 1)")
            expDictionary.Add("polyline_Get_X_AtPoint", "Polyline|Number|Y|1|Point Along (0  to 1)")
            expDictionary.Add("polyline_Get_Y_AtPoint", "Polyline|Number|Y|1|Point Along (0  to 1)")
            expDictionary.Add("polyline_Get_Z_AtPoint", "Polyline|Number|N|1|Point Along (0  to 1)")
            expDictionary.Add("polyline_Get_M_AtPoint", "Polyline|Number|N|1|Point Along (0  to 1)")
            expDictionary.Add("shape_CountVertices", "All|Number|N|0")
            expDictionary.Add("shape_CountAllParts", "All|Number|N|0")
            expDictionary.Add("shape_Get_X_Min", "All|Number|Y|0")
            expDictionary.Add("shape_Get_X_Max", "All|Number|Y|0")
            expDictionary.Add("shape_Get_Y_Min", "All|Number|Y|0")
            expDictionary.Add("shape_Get_Y_Max", "All|Number|Y|0")
            expDictionary.Add("shape_Get_M_Min", "All|Number|N|0")
            expDictionary.Add("shape_Get_M_Max", "All|Number|N|0")
            expDictionary.Add("shape_Get_Z_Min", "All|Number|N|0")
            expDictionary.Add("shape_Get_Z_Max", "All|Number|N|0")
            'Example for a new function
            'Can be applied to Polylines and Polygons
            'The result will be stored in numeric field
            'Spatial reference will be used
            'No parameters are needed
            'expDictionary.Add("shape_Get_Length", "PgPl|Number|Y|0")
            Return expDictionary
        Catch
            Return Nothing
        End Try
    End Function
    Public Function GetGeometryExpressionsDescription() As Dictionary(Of String, String)
        Dim expDictionary As Dictionary(Of String, String) = New Dictionary(Of String, String)
        Try
            expDictionary.Add("point_Get_X_DMS", "Populates a string field of the Point attribute table with the longitude of the points in Degrees Minutes Seconds. If the data is in a Projected Coordinate System (PCS), the results will be in the Geographic Coordinate System (GCS) of this PCS." & vbNewLine & vbNewLine & _
                              "Layer Type = Point" & vbNewLine & vbNewLine & _
                              "Field Type = String" & vbNewLine & vbNewLine & _
                              "Parameter OutputType: - DMS for Degrees Minutes Seconds or DM for Degrees Minutes." & vbNewLine & _
                              "Parameter Precision: - a number indicating the number of digits after the decimal point.")
            expDictionary.Add("point_Get_Y_DMS", "Populates a string field of the Point attribute table with the latitude of the points in Degrees Minutes Seconds. If the data is in a Projected Coordinate System (PCS), the results will be in the Geographic Coordinate System (GCS) of this PCS." & vbNewLine & vbNewLine & _
                              "Layer Type = Point" & vbNewLine & vbNewLine & _
                              "Field Type = String" & vbNewLine & vbNewLine & _
                              "Parameter OutputType: - DMS for Degrees Minutes Seconds or DM for Degrees Minutes." & vbNewLine & _
                              "Parameter Precision: - a number indicating the number of digits after the decimal point.")
            expDictionary.Add("point_Get_M", "Populates a field with the M values of the points from a PointM layer. If applied on point dataset that does not have M all records will be populated with 0." & vbNewLine & vbNewLine & _
                              "Layer Type = PointM" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("point_Get_X", "Populates a field with the X values of the points from a Point layer. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = Point" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used")
            expDictionary.Add("point_Get_Y", "Populates a field with the Y values of the points from a Point layer. The spatial reference of the dataset or the Map can be used.." & vbNewLine & vbNewLine & _
                              "Layer Type = Point" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used")
            expDictionary.Add("point_Get_Z", "Populates a field with the Z values of the points from a PointZ layer. If applied on point dataset that does not have Z all records will be populated with 0." & vbNewLine & vbNewLine & _
                              "Layer Type = PointZ" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("polygon_CountTrueParts", "Calculate the count of true parts (holes not considered as parts) of the polygoins." & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("polygon_Get_X_Center", "Populates a field with the X values of the centroid (can be outside of the polygon) of the polygons from a polygon layer. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used")
            expDictionary.Add("polygon_Get_Y_Center", "Populates a field with the Y values of the centroid (can be outside of the polygon) of the polygons from a polygon layer. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used")
            expDictionary.Add("polygon_Get_X_Label", "Populates a field with the X values of the label (always inside of the polygon) points of the polygons from a polygon layer. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used")
            expDictionary.Add("polygon_Get_Y_Label", "Populates a field with the Y values of the label (always inside of the polygon) points of the polygons from a polygon layer. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used")
            expDictionary.Add("polygon_Return_Area", "Calculates the area of the polygons. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon" & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & _
                              "Spatial Reference = Used" & vbNewLine & vbNewLine & _
                              "Parameter Output Units: - The units in which the area will be calculated. Note that this will be used only if the selected spatial reference is a Projected Coordinate System. The output units can be Acres, Square Kilometers, Square Meters, Square Miles, Square Yards, Square Feet, Hectares")
            expDictionary.Add("polygon_Thickness", "Calculates the thickness ratio of polygons. Can be used to identify skinny polygons. For polygons with close to circular shape the result will be close to 1. The thinner the polygons are the smaller the ratio will be." & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("polyline_Get_Azimuth", "Calculates the azimuth of the polylines at a point. The azimuth is stored in decimal degrees 0 = North, 90 = East, 180 = south, 270 = West." & vbNewLine & vbNewLine & _
                              "Layer Type = Polyline" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Point Along: - specified with its relative location along the polyline (0 = start, 0.5 = middle, 1 = end)")
            expDictionary.Add("polyline_Get_X_AtPoint", "Calculates the X coordinate of a point along the polyline. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = Polyline" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used" & vbNewLine & vbNewLine & _
                              "Parameter Point Along: - specified with its relative location along the polyline (0 = FROM Point, 0.5 = Middle Point, 1 = TO Point)")
            expDictionary.Add("polyline_Get_Y_AtPoint", "Calculates the Y coordinate of a point along the polyline. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = Polyline" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & _
                              "Spatial Reference = Used" & vbNewLine & vbNewLine & _
                              "Parameter Point Along: - specified with its relative location along the polyline (0 = FROM Point, 0.5 = Middle Point, 1 = TO Point)")
            expDictionary.Add("polyline_Get_Z_AtPoint", "Calculates the Z of a point along a PolylineZ. If the polyline is not Z enabled will return 0." & vbNewLine & vbNewLine & _
                              "Layer Type = Polyline" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & _
                              "Spatial Reference = NotUsed" & vbNewLine & vbNewLine & _
                              "Parameter Point Along: - specified with its relative location along the polyline (0 = FROM Point, 0.5 = Middle Point, 1 = TO Point)")
            expDictionary.Add("polyline_Get_M_AtPoint", "Calculates the M of a point along a PolylineM. If the polyline is not Z enabled will return 0." & vbNewLine & vbNewLine & _
                              "Layer Type = Polyline" & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & _
                              "Spatial Reference = NotUsed" & vbNewLine & _
                              "Parameter Point Along: - specified with its relative location along the polyline (0 = FROM Point, 0.5 = Middle Point, 1 = TO Point)")
            expDictionary.Add("shape_CountVertices", "Calculate the count of the vertices of each feature." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("shape_CountAllParts", "Calculate the number pf parts in each feature." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("shape_Get_X_Min", "Populates a field with the minimum X value of the shape. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used")
            expDictionary.Add("shape_Get_X_Max", "Populates a field with the maximum X value of the shape. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used")
            expDictionary.Add("shape_Get_Y_Min", "Populates a field with the minimum Y value of the shape. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used")
            expDictionary.Add("shape_Get_Y_Max", "Populates a field with the maximum Y value of the shape. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used")
            expDictionary.Add("shape_Get_M_Min", "Populates a field with the minimum M value of the shape." & vbNewLine & vbNewLine & _
                             "Layer Type = All" & vbNewLine & vbNewLine & _
                             "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                             "Spatial Reference = Not Used")
            expDictionary.Add("shape_Get_M_Max", "Populates a field with the maximum M value of the shape." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("shape_Get_Z_Min", "Populates a field with the minimum Z value of the shape." & vbNewLine & vbNewLine & _
                             "Layer Type = All" & vbNewLine & vbNewLine & _
                             "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                             "Spatial Reference = Not Used")
            expDictionary.Add("shape_Get_Z_Max", "Populates a field with the maximum Z value of the shape." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number (Double recommended)" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            'Example for a new function
            'expDictionary.Add("shape_Get_Length", "Calculates the length of polylines and perimeter of polygons. The spatial reference of the dataset or the Map can be used." & vbNewLine & vbNewLine & _
            '                  "Layer Type = Polygon & Polyline" & vbNewLine & _
            '                  "Field Type = Number (Double recommended)" & vbNewLine & _
            '                  "Spatial Reference = Used")
            Return expDictionary
        Catch
            Return Nothing
        End Try
    End Function

    Public Function GetDrawExpressions() As Dictionary(Of String, String)
        Dim expDictionary As Dictionary(Of String, String) = New Dictionary(Of String, String)
        Try
            expDictionary.Add("feature_To_Graphic", "All|Number|N|0")
            expDictionary.Add("draw_Bounding_Rectangle", "PgPlMp|Number|N|0")
            expDictionary.Add("draw_ConvexHull", "PgPlMp|Number|N|0")
            expDictionary.Add("draw_Vertices", "PgPlMp|Number|N|0")
            expDictionary.Add("draw_Buffer", "All|Number|Y|1|Buffer Distance")
            Return expDictionary
        Catch
            Return Nothing
        End Try
    End Function
    Public Function GetDrawExpressionsDescription() As Dictionary(Of String, String)
        Dim expDictionary As Dictionary(Of String, String) = New Dictionary(Of String, String)
        Try
            expDictionary.Add("feature_To_Graphic", "Draws with the current graphic symbol a graphic shape for each feature of the selected layer. If a field is selected the result will be recorded in the attribute table (1 = success, 0 = failure)" & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("draw_Bounding_Rectangle", "Draws with the current graphic symbol a graphic shape representing the bounding rectangle of each feature of the selected layer. If a field is selected the result will be recorded in the attribute table (1 = success, 0 = failure)" & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon, Polyline, Multipoint" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("draw_ConvexHull", "Draws with the current graphic symbol a graphic shape representing the convex hull of each feature of the selected layer. If a field is selected the result will be recorded in the attribute table (1 = success, 0 = failure)" & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon, Polyline, Multipoint" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("draw_Vertices", "Draws with the current graphic symbol graphic points representing the vertices of the features of the selected layer. If a field is selected the result will be recorded in the attribute table (1 = success, 0 = failure)" & vbNewLine & vbNewLine & _
                             "Layer Type = Polygon, Polyline, Multipoint" & vbNewLine & vbNewLine & _
                             "Field Type = Number" & vbNewLine & vbNewLine & _
                             "Spatial Reference = Not Used")
            expDictionary.Add("draw_Buffer", "Draws with the current graphic symbol a graphic of each feature of the selected layer buffered with the specified buffer distance." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Used" & vbNewLine & vbNewLine & _
                              "Parameter Buffer Distance: in the units of the selected spatial reference.")
            Return expDictionary
        Catch
            Return Nothing
        End Try
    End Function
    Public Function GetEditExpressions() As Dictionary(Of String, String)
        Dim expDictionary As Dictionary(Of String, String) = New Dictionary(Of String, String)
        Try
            expDictionary.Add("edit_SimplifyGeometry", "All|Number|N|0")
            expDictionary.Add("edit_Generalize", "All|Number|N|1|Generalize Tolerance")
            expDictionary.Add("edit_Densify", "All|Number|N|1|Max segment length")
            expDictionary.Add("edit_ClosePolylines", "Polyline|Number|N|1|Max Distance To Close")
            expDictionary.Add("edit_PolygonFillHoles", "Polygon|Number|N|0")
            expDictionary.Add("edit_PolygonScale", "Polygon|Number|N|2|X Scale Factor|Y Scale Factor")
            expDictionary.Add("edit_PolygonRotate", "Polygon|Number|N|1|Rotation Angle")
            expDictionary.Add("edit_PolylineFlip", "Polyline|Number|N|0")
            expDictionary.Add("edit_PolylineRotate", "Polyline|Number|N|2|Point Location|Rotation Angle")
            expDictionary.Add("edit_PolylineOffset", "Polyline|Number|N|1|Offset Distance")
            expDictionary.Add("edit_OffsetZs", "All|Number|N|1|Offset Distance")
            expDictionary.Add("edit_MultiplyZs", "All|Number|N|1|Multiply Value")
            expDictionary.Add("edit_SetConstantZ", "All|Number|N|1|Z Value")
            expDictionary.Add("edit_OffsetMs", "All|Number|N|1|Offset Distance")
            expDictionary.Add("edit_MultiplyMs", "All|Number|N|1|Multiply Value")
            Return expDictionary
        Catch
            Return Nothing
        End Try
    End Function
    Public Function GetEditExpressionsDescription() As Dictionary(Of String, String)
        Dim expDictionary As Dictionary(Of String, String) = New Dictionary(Of String, String)
        Try
            expDictionary.Add("edit_SimplifyGeometry", "Simplifies the geometry of the features. For example if a polygon has incorrect orientation of the rings, this expression will fix it." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("edit_Generalize", "Generalizes (removes excess vertices) from polylines or polygons. Notes:1. If applied to polygons some Gaps and/or Overlaps will be created on the boundaries of the adjacen polygons!!!!!!" & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Generalize Tolerance: The tolerance in the units of the spatial reference of the dataset.")
            expDictionary.Add("edit_Densify", "Densifies (inserts new vertices vertices) to the polylines or polygons. Notes:1. If applied to polygons some Gaps and/or Overlaps will be created on the boundaries of the adjacen polygons!!!!!!" & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Max segment length: Distance between the vertices in the units of the spatial reference of the dataset.")
            expDictionary.Add("edit_ClosePolylines", "Closes the polylines by adding a new segment connecting the From and To nodes. The user needs to specify the max distance to be closed." & vbNewLine & vbNewLine & _
                              "Layer Type = Polyline" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Max Distance To Close: Distances between nodes larger than this tolerances will not be closed.")
            expDictionary.Add("edit_PolygonFillHoles", "Fills the holes in the polygons." & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("edit_PolygonScale", "Scales the polygons with user specified X & Y scale factors. The origin point of the scale is the centroid of the polygons." & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter X Scale Factor" & vbNewLine & vbNewLine & _
                              "Parameter Y Scale Factor")
            expDictionary.Add("edit_PolygonRotate", "Rotates the polygons around the centre point with user specified rotation angle." & vbNewLine & vbNewLine & _
                              "Layer Type = Polygon" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Rotation Angle")
            expDictionary.Add("edit_PolylineRotate", "Rotates the polygons around a point specified by the user with its location along the polyline with user specified rotation angle." & vbNewLine & vbNewLine & _
                              "Layer Type = Polyline" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Point location - (0 = FROM Point, 0.5 = Middle Point, 1 = TO Point)" & vbNewLine & vbNewLine & _
                              "Parameter Rotation Angle")
            expDictionary.Add("edit_PolylineFlip", "Flips (Reverses the orientation) of the polylines." & vbNewLine & vbNewLine & _
                              "Layer Type = Polyline" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used")
            expDictionary.Add("edit_PolylineOffset", "Offsets the polylines with user specified offset distance." & vbNewLine & vbNewLine & _
                              "Layer Type = Polyline" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Offset Distance: Positive values - offset right, Negative values - offset left.")
            expDictionary.Add("edit_OffsetZs", "Offsets the Z values with user specified offset distance." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Offset Distance.")
            expDictionary.Add("edit_MultiplyZs", "Multiplies the Z values with user specified value." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Multiply Value.")
            expDictionary.Add("edit_SetConstantZ", "Sets a constant Z values for each shape." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Z Value.")
            expDictionary.Add("edit_OffsetMs", "Offsets the M values with user specified offset distance." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Offset Distance.")
            expDictionary.Add("edit_MultiplyMs", "Multiplies the M values with user specified value." & vbNewLine & vbNewLine & _
                              "Layer Type = All" & vbNewLine & vbNewLine & _
                              "Field Type = Number" & vbNewLine & vbNewLine & _
                              "Spatial Reference = Not Used" & vbNewLine & vbNewLine & _
                              "Parameter Multiply Value.")
            Return expDictionary
        Catch
            Return Nothing
        End Try
    End Function

End Module
