﻿Option Strict On
Option Explicit On

Imports ESRI.ArcGIS.ArcMapUI
Imports ESRI.ArcGIS.Carto
Imports ESRI.ArcGIS.Geodatabase
Imports ESRI.ArcGIS.esriSystem
Imports ESRI.ArcGIS.Editor
Imports ESRI.ArcGIS.Geometry
Imports ESRI.ArcGIS.Display
Imports System.Math
'Structure used to pass the parameters
Public Structure expGeometryParams
    Dim Geometry As ESRI.ArcGIS.Geometry.IGeometry
    Dim DataSref As ESRI.ArcGIS.Geometry.ISpatialReference
    Dim MapSref As ESRI.ArcGIS.Geometry.ISpatialReference
    Dim SrefFromData As Boolean
    Dim Par1 As String
    Dim Par2 As String
    Dim Par3 As String
End Structure

Module SupportFunctions
    Public params As expGeometryParams
    Public Function EC_GetFeatureLayers(ByVal sTyp As String) As Collection
        Dim pMxDoc As IMxDocument = My.ArcMap.Document
        Dim pMap As IMap = pMxDoc.FocusMap
        Dim pFLayer As IFeatureLayer = Nothing
        Dim lCol As New Collection
        Dim pEnumLayer As IEnumLayer = Nothing
        Dim pUID As UID = Nothing
        Try
            If (pMap.LayerCount = 0) Then
                Return Nothing
            End If
            pUID = New UID
            pUID.Value = "{E156D7E5-22AF-11D3-9F99-00C04F6BC78E}"
            pEnumLayer = pMap.Layers(pUID, True)
            pEnumLayer.Reset()
            pFLayer = CType(pEnumLayer.Next, IFeatureLayer)
            Do While Not pFLayer Is Nothing
                If pFLayer.Valid Then
                    If TypeOf pFLayer Is IFeatureLayer Then
                        If pFLayer.FeatureClass.FeatureType <> esriFeatureType.esriFTAnnotation Then
                            Select Case sTyp
                                Case "Polygon"
                                    If pFLayer.FeatureClass.ShapeType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                                        lCol.Add(pFLayer.Name)
                                    End If
                                Case "Polyline"
                                    If pFLayer.FeatureClass.ShapeType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline Then
                                        lCol.Add(pFLayer.Name)
                                    End If
                                Case "PgPl"
                                    If pFLayer.FeatureClass.ShapeType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline Or _
                                        pFLayer.FeatureClass.ShapeType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Then
                                        lCol.Add(pFLayer.Name)
                                    End If
                                Case "PgPlMp"
                                    If pFLayer.FeatureClass.ShapeType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolyline Or _
                                        pFLayer.FeatureClass.ShapeType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPolygon Or _
                                        pFLayer.FeatureClass.ShapeType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryMultipoint Then
                                        lCol.Add(pFLayer.Name)
                                    End If
                                Case "Point"
                                    If pFLayer.FeatureClass.ShapeType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryPoint Then
                                        lCol.Add(pFLayer.Name)
                                    End If
                                Case "Multipoint"
                                    If pFLayer.FeatureClass.ShapeType = ESRI.ArcGIS.Geometry.esriGeometryType.esriGeometryMultipoint Then
                                        lCol.Add(pFLayer.Name)
                                    End If
                                Case "All"
                                    lCol.Add(pFLayer.Name)
                            End Select
                        End If
                    End If
                End If
                pFLayer = CType(pEnumLayer.Next, IFeatureLayer)
            Loop
            Return lCol
        Catch ex As Exception
            MsgBox("Error in EC_GetFeatureLayers. Error:" & ex.Message)
            Return Nothing
        Finally
            System.Runtime.InteropServices.Marshal.ReleaseComObject(pMxDoc)
        End Try
    End Function
    Friend Function EC_GetLayer(ByRef etLayName As String) As ESRI.ArcGIS.Carto.IFeatureLayer
        Dim pMxDoc As IMxDocument = My.ArcMap.Document
        Dim pMap As IMap = pMxDoc.FocusMap
        Dim pFLayer As IFeatureLayer = Nothing
        Dim pSelLayer As IFeatureLayer = Nothing
        Dim pEnumLayer As IEnumLayer = Nothing
        Dim pUID As UID = Nothing
        Try
            pUID = New UID
            pUID.Value = "{E156D7E5-22AF-11D3-9F99-00C04F6BC78E}"
            pEnumLayer = pMap.Layers(pUID, True)
            pEnumLayer.Reset()
            pFLayer = CType(pEnumLayer.Next, IFeatureLayer)
            Do While Not pFLayer Is Nothing
                If TypeOf pFLayer Is IFeatureLayer Then
                    If (pFLayer.Name = etLayName) Then
                        pSelLayer = pFLayer
                        Return pSelLayer
                    End If
                End If
                pFLayer = CType(pEnumLayer.Next, IFeatureLayer)
            Loop
            Return pSelLayer
        Catch ex As Exception
            MsgBox("Error in EC_GetLayer - Error:" & ex.Message)
            Return Nothing
        Finally
            System.Runtime.InteropServices.Marshal.ReleaseComObject(pMxDoc)
        End Try

    End Function
    Friend Function EC_GetEditor() As IEditor
        Dim pEditor As IEditor
        Dim pUID As New UID
        Try
            pUID.Value = "esriEditor.Editor"
            pEditor = CType(My.ArcMap.Application.FindExtensionByCLSID(pUID), IEditor)
            Return pEditor
        Catch
            Return Nothing
        End Try
    End Function
    Public Function EC_GetStep(ByVal lCount As Long) As Integer
        Dim iStep As Integer
        Try
            If lCount <= 100 Then
                iStep = 1
            ElseIf lCount > 100 And lCount <= 1000 Then
                iStep = 10
            ElseIf lCount > 1000 And lCount <= 10000 Then
                iStep = 100
            ElseIf lCount > 10000 And lCount <= 100000 Then
                iStep = 1000
            ElseIf lCount > 100000 Then
                iStep = 10000
            End If
            Return iStep
        Catch
            Return 1
        End Try
    End Function
    Public Function EC_DD2DMS(ByVal dDD As Double, ByVal sWhat As String, ByVal sType As String, ByVal iNumDec As Short) As String
        Dim dD As Double, dM As Double, dS As Double
        Dim dM1 As Double, dS1 As Double
        Dim sD As String, sM As String, sS As String
        Dim sSuf As String
        Dim sDMS As String
        Dim sDeg As String = Chr(176)
        Dim sMin As String = Chr(39)
        Dim sSec As String = Chr(34)
        Try
            If sWhat = "Lon" Then
                If dD >= 0 Then
                    sSuf = "E"
                Else
                    sSuf = "W"
                End If
            Else
                If dD >= 0 Then
                    sSuf = "N"
                Else
                    sSuf = "S"
                End If
            End If

            dDD = Abs(dDD)
            dD = Int(dDD)
            sD = CStr(dD)
            dM = (dDD - dD) * 60
            dM1 = Int(dM)
            If (sType = "DMS") Then
                If (Len(CStr(dM1)) = 1) Then
                    sM = "0" & CStr(dM1)
                Else
                    sM = CStr(dM1)
                End If
                dS = CDbl(FormatNumber(((dM - dM1) * 60), iNumDec))
                dS1 = Int(dS)
                If (Len(CStr(dS1)) = 1) Then
                    sS = "0" & CStr(dS)
                Else
                    sS = CStr(dS)
                End If
                sDMS = sD & sDeg & sM & sMin & sS & sSec & sSuf
            Else
                sM = CStr(FormatNumber(dM, iNumDec))
                sDMS = sD & sDeg & sM & sMin & sSuf
            End If
            Return sDMS
        Catch
            Return String.Empty
        End Try
    End Function
    Friend Function EC_ConvertArea(ByRef dAreaMeters As Double, ByRef sOutUnits As String) As Double
        Dim dC As Double
        Select Case UCase(sOutUnits)
            Case "ACRES"
                dC = 0.00024710538
            Case "SQUARE KILOMETERS"
                dC = 0.000001
            Case "SQUARE METERS"
                dC = 1
            Case "SQUARE MILES"
                dC = 0.0000003861022
            Case "SQUARE YARDS"
                dC = 1.19599
            Case "SQUARE FEET"
                dC = 10.76386736111
            Case "HECTARES"
                dC = 0.0001
            Case Else
                dC = 1
        End Select
        Return (dAreaMeters * dC)
    End Function
    Public Function ET_DrawPointGraphic(ByVal pPoint As IPoint) As Short
        Dim pMxDoc As IMxDocument = My.ArcMap.Document
        Dim pMap As IMap = pMxDoc.FocusMap
        Dim pContainer As IGraphicsContainer = CType(pMxDoc.FocusMap, IGraphicsContainer)
        Dim pElement As IElement
        Dim pPointElement As IMarkerElement
        Dim pElProps As IElementProperties
        Dim pMarker As IMarkerSymbol = CType(ET_GetDefaultSymbol("Point"), IMarkerSymbol)
        Try
            If Not pPoint Is Nothing Then
                pPointElement = New MarkerElement
                pPointElement.Symbol = pMarker
                pElement = CType(pPointElement, IElement)
                pElement.Geometry = pPoint
                pElProps = CType(pPointElement, IElementProperties)
                pContainer.AddElement(pElement, 0)
            End If
            Return 1
        Catch
            Return 0
        Finally
            System.Runtime.InteropServices.Marshal.ReleaseComObject(pMxDoc)
        End Try
    End Function

    Public Function ET_DrawPolygonGraphic(ByVal pPolygon As IPolygon) As Short
        Dim pMxDoc As IMxDocument = My.ArcMap.Document
        Dim pMap As IMap = pMxDoc.FocusMap
        Dim pFillSymbol As IFillSymbol = CType(ET_GetDefaultSymbol("Polygon"), IFillSymbol)
        Dim pContainer As IGraphicsContainer = CType(pMap, IGraphicsContainer)
        Dim pElement As IElement
        Dim pPolEl As IFillShapeElement
        Dim pElProps As IElementProperties
        Dim pTopOp As ITopologicalOperator
        Try
            If Not pPolygon Is Nothing Then
                pTopOp = CType(pPolygon, ITopologicalOperator)
                pTopOp.Simplify()
                pPolEl = New PolygonElement
                pPolEl.Symbol = pFillSymbol
                pElement = CType(pPolEl, IElement)
                pElement.Geometry = pPolygon
                pElProps = CType(pPolEl, IElementProperties)
                pContainer.AddElement(pElement, 0)
            End If
            Return 1
        Catch
            Return 0
        Finally
            System.Runtime.InteropServices.Marshal.ReleaseComObject(pMxDoc)
        End Try
    End Function
    Public Function ET_DrawPolylineGraphic(ByVal pPl As IPolyline) As Short
        Dim pMxDoc As IMxDocument = My.ArcMap.Document
        Dim pMap As IMap = pMxDoc.FocusMap
        Dim pLineSymbol As ILineSymbol = CType(ET_GetDefaultSymbol("Polyline"), ILineSymbol)
        Dim pContainer As IGraphicsContainer = CType(pMap, IGraphicsContainer)
        Dim pElement As IElement
        Dim pPolylineElement As ILineElement
        Dim pElProps As IElementProperties
        Try
            If Not pPl Is Nothing Then
                pPolylineElement = New LineElement
                pPolylineElement.Symbol = pLineSymbol
                pElement = CType(pPolylineElement, IElement)
                pElement.Geometry = pPl
                pElProps = CType(pPolylineElement, IElementProperties)
                pContainer.AddElement(pElement, 0)
            End If
            Return 1
        Catch
            Return 0
        Finally
            System.Runtime.InteropServices.Marshal.ReleaseComObject(pMxDoc)
        End Try
    End Function
    Friend Function ET_GetDefaultSymbol(ByVal sType As String) As ISymbol
        Dim pMxDoc As IMxDocument = My.ArcMap.Document
        Dim pDefaults As IBasicDocumentDefaultSymbols
        Dim pSym As ISymbol = Nothing
        Try
            pDefaults = CType(pMxDoc, IBasicDocumentDefaultSymbols)
            Select Case sType
                Case "Point"
                    pSym = CType(pDefaults.MarkerSymbol, ISymbol)
                Case "Polyline"
                    pSym = CType(pDefaults.LineSymbol, ISymbol)
                Case "Polygon"
                    pSym = CType(pDefaults.FillSymbol, ISymbol)
            End Select
            Return pSym
        Catch
            Return Nothing
        Finally
            System.Runtime.InteropServices.Marshal.ReleaseComObject(pMxDoc)
        End Try
    End Function
End Module
